@@ -1,8 +1,11 @@
 /// To run an example run the following command
 /// `cargo run --example interact`.
+
+#[cfg(unix)]
 use expectrl::repl::spawn_bash;
 
-#[cfg(feature = "sync")]
+#[cfg(unix)]
+#[cfg(not(feature = "async"))]
 fn main() {
     let mut bash = spawn_bash().expect("Error while spawning bash");
 
@@ -14,6 +17,7 @@ fn main() {
     println!("Quiting status {:?}", status);
 }
 
+#[cfg(unix)]
 #[cfg(feature = "async")]
 fn main() {
     let mut bash = futures_lite::future::block_on(spawn_bash()).expect("Error while spawning bash");
@@ -25,3 +29,15 @@ fn main() {
 
     println!("Quiting status {:?}", status);
 }
+
+#[cfg(windows)]
+fn main() {
+    let mut pwsh = expectrl::spawn("cmd").expect("Error while spawning bash");
+
+    println!("Now you're in interacting mode");
+    println!("To return control back to main type CTRL-]");
+
+    pwsh.interact().expect("Failed to start interact");
+
+    println!("Quiting");
+}
