@@ -1,7 +1,7 @@
-#[cfg(feature = "sync")]
+#[cfg(not(feature = "async"))]
 #[cfg(feature = "log")]
 fn main() -> Result<(), expectrl::Error> {
-    let mut p = expectrl::Session::spawn("cat")?;
+    let mut p = expectrl::spawn("cat")?;
     p.set_log(std::io::stdout());
     p.send_line("Hello World")?;
 
@@ -12,7 +12,7 @@ fn main() -> Result<(), expectrl::Error> {
     Ok(())
 }
 
-#[cfg(not(feature = "log"))]
+#[cfg(any(not(feature = "log"), feature = "async"))]
 fn main() {
     println!(
         "To run the example set necessary features `--no-default-features --features log,sync`"
