@@ -1,16 +1,15 @@
 use expectrl::{repl::spawn_python, Regex};
 
-#[cfg(feature = "sync")]
+#[cfg(not(feature = "async"))]
 fn main() {
     let mut p = spawn_python().unwrap();
 
     p.execute("import platform").unwrap();
     p.send_line("platform.node()").unwrap();
 
-    // todo: add support for matches in 'Found' + iterator?
-    let found = p.expect(Regex(r"'\w+'")).unwrap();
+    let found = p.expect(Regex(r"'.*'")).unwrap();
 
-    println!("Platform {}", String::from_utf8_lossy(found.found_match()));
+    println!("Platform {}", String::from_utf8_lossy(found.first()));
 }
 
 #[cfg(feature = "async")]
@@ -21,9 +20,8 @@ fn main() {
         p.execute("import platform").await.unwrap();
         p.send_line("platform.node()").await.unwrap();
 
-        // todo: add support for matches in 'Found' + iterator?
-        let found = p.expect(Regex(r"'\w+'")).await.unwrap();
+        let found = p.expect(Regex(r"'.*'")).await.unwrap();
 
-        println!("Platform {}", String::from_utf8_lossy(found.found_match()));
+        println!("Platform {}", String::from_utf8_lossy(found.first()));
     })
 }
