@@ -1,5 +1,4 @@
-#![cfg(any(feature = "log", feature = "async_log"))]
-//! A wrapper of Session to log a read/write operations
+//! A wrapper of [Session] to log a read/write operations
 
 use crate::{error::Error, session::Session};
 use std::{
@@ -11,26 +10,15 @@ use std::{
 /// A logging wrapper of session
 pub struct SessionWithLog {
     inner: Session,
-    logger: Option<Box<dyn Write>>,
+    logger: Option<Box<dyn Write + Send>>,
 }
 
 impl SessionWithLog {
-    /// Spawn a session wrapped with logger.
-    ///
-    /// See [Session].
-    pub fn spawn(cmd: &str) -> Result<Self, Error> {
-        let session = Session::spawn(cmd)?;
-        Ok(Self {
-            inner: session,
-            logger: None,
-        })
-    }
-
     /// Spawn session wrapped with logger.
     ///
     /// See [Session].
-    pub fn spawn_cmd(cmd: Command) -> Result<Self, Error> {
-        let session = Session::spawn_cmd(cmd)?;
+    pub fn spawn(cmd: Command) -> Result<Self, Error> {
+        let session = Session::spawn(cmd)?;
         Ok(Self {
             inner: session,
             logger: None,
@@ -40,7 +28,7 @@ impl SessionWithLog {
     /// Set a writer for which is used for logging.
     ///
     /// Logger is suppose to be called on all IO operations.
-    pub fn set_log<W: Write + 'static>(&mut self, w: W) {
+    pub fn set_log<W: Write + Send + 'static>(&mut self, w: W) {
         self.logger = Some(Box::new(w));
     }
 
@@ -54,7 +42,7 @@ impl SessionWithLog {
     }
 }
 
-#[cfg(feature = "log")]
+#[cfg(all(feature = "log", not(feature = "async")))]
 impl SessionWithLog {
     pub fn send<S: AsRef<str>>(&mut self, s: S) -> io::Result<()> {
         self.log("send", s.as_ref().as_bytes());
@@ -67,7 +55,7 @@ impl SessionWithLog {
     }
 }
 
-#[cfg(feature = "async_log")]
+#[cfg(feature = "async")]
 impl SessionWithLog {
     pub async fn send<S: AsRef<str>>(&mut self, s: S) -> io::Result<()> {
         self.log("send", s.as_ref().as_bytes());
@@ -94,7 +82,7 @@ impl DerefMut for SessionWithLog {
     }
 }
 
-#[cfg(feature = "log")]
+#[cfg(not(feature = "async"))]
 impl std::io::Write for SessionWithLog {
     fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
         self.log("write", buf);
@@ -106,7 +94,7 @@ impl std::io::Write for SessionWithLog {
     }
 }
 
-#[cfg(feature = "log")]
+#[cfg(not(feature = "async"))]
 impl std::io::Read for SessionWithLog {
     fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
         let result = self.deref_mut().read(buf);
@@ -118,7 +106,7 @@ impl std::io::Read for SessionWithLog {
     }
 }
 
-#[cfg(feature = "log")]
+#[cfg(not(feature = "async"))]
 impl std::io::BufRead for SessionWithLog {
     fn fill_buf(&mut self) -> io::Result<&[u8]> {
         self.inner.fill_buf()
@@ -142,33 +130,33 @@ impl std::io::BufRead for SessionWithLog {
     }
 }
 
-#[cfg(feature = "async_log")]
+#[cfg(feature = "async")]
 impl futures_lite::io::AsyncWrite for SessionWithLog {
     fn poll_write(
         mut self: std::pin::Pin<&mut Self>,
         cx: &mut std::task::Context<'_>,
         buf: &[u8],
     ) -> std::task::Poll<std::io::Result<usize>> {
-        self.log("write", &buf);
-        std::pin::Pin::new(self.inner.deref_mut().deref_mut()).poll_write(cx, buf)
+        self.log("write", buf);
+        std::pin::Pin::new(&mut self.inner).poll_write(cx, buf)
     }
 
     fn poll_flush(
         mut self: std::pin::Pin<&mut Self>,
         cx: &mut std::task::Context<'_>,
     ) -> std::task::Poll<std::io::Result<()>> {
-        std::pin::Pin::new(self.inner.deref_mut().deref_mut()).poll_flush(cx)
+        std::pin::Pin::new(&mut self.inner).poll_flush(cx)
     }
 
     fn poll_close(
         mut self: std::pin::Pin<&mut Self>,
         cx: &mut std::task::Context<'_>,
     ) -> std::task::Poll<std::io::Result<()>> {
-        std::pin::Pin::new(self.inner.deref_mut().deref_mut()).poll_close(cx)
+        std::pin::Pin::new(&mut self.inner).poll_close(cx)
     }
 }
 
-#[cfg(feature = "async_log")]
+#[cfg(feature = "async")]
 impl futures_lite::io::AsyncRead for SessionWithLog {
     fn poll_read(
         mut self: std::pin::Pin<&mut Self>,
@@ -176,7 +164,7 @@ impl futures_lite::io::AsyncRead for SessionWithLog {
         buf: &mut [u8],
     ) -> std::task::Poll<std::io::Result<usize>> {
         let result = futures_lite::io::AsyncRead::poll_read(
-            std::pin::Pin::new(self.inner.deref_mut().deref_mut()), // haven't foudn any better way
+            std::pin::Pin::new(&mut self.inner), // haven't foudn any better way
             cx,
             buf,
         );
@@ -189,7 +177,23 @@ impl futures_lite::io::AsyncRead for SessionWithLog {
     }
 }
 
-#[cfg(feature = "async_log")]
+#[cfg(feature = "async")]
+impl futures_lite::io::AsyncBufRead for SessionWithLog {
+    fn poll_fill_buf(
+        self: std::pin::Pin<&mut Self>,
+        cx: &mut std::task::Context<'_>,
+    ) -> std::task::Poll<std::io::Result<&[u8]>> {
+        let this = self.get_mut();
+        let proc = std::pin::Pin::new(&mut this.inner);
+        proc.poll_fill_buf(cx)
+    }
+
+    fn consume(mut self: std::pin::Pin<&mut Self>, amt: usize) {
+        std::pin::Pin::new(&mut self.inner).consume(amt);
+    }
+}
+
+#[cfg(feature = "async")]
 impl SessionWithLog {
     /// The function behaives in the same way as [futures_lite::io::AsyncBufReadExt].
     ///
@@ -218,18 +222,19 @@ mod test {
     use super::*;
     use std::{
         io::Cursor,
+        process::Command,
         sync::{Arc, Mutex},
         thread,
         time::Duration,
     };
 
-    #[cfg(feature = "sync")]
+    #[cfg(not(feature = "async"))]
     #[cfg(feature = "log")]
     #[test]
     fn log() {
         use std::io::Read;
 
-        let mut session = SessionWithLog::spawn("cat").unwrap();
+        let mut session = SessionWithLog::spawn(Command::new("cat")).unwrap();
         let writer = StubWriter::default();
         session.set_log(writer.clone());
         session.send_line("Hello World").unwrap();
@@ -248,13 +253,13 @@ mod test {
         )
     }
 
-    #[cfg(feature = "sync")]
+    #[cfg(not(feature = "async"))]
     #[cfg(feature = "log")]
     #[test]
     fn log_read_line() {
         use std::io::BufRead;
 
-        let mut session = SessionWithLog::spawn("cat").unwrap();
+        let mut session = SessionWithLog::spawn(Command::new("cat")).unwrap();
         let writer = StubWriter::default();
         session.set_log(writer.clone());
         session.send_line("Hello World").unwrap();
@@ -271,14 +276,13 @@ mod test {
         )
     }
 
-    #[cfg(feature = "async_log")]
-    #[cfg(feature = "async")]
+    #[cfg(all(feature = "async", feature = "log"))]
     #[test]
     fn log() {
         use futures_lite::AsyncReadExt;
 
         futures_lite::future::block_on(async {
-            let mut session = SessionWithLog::spawn("cat").unwrap();
+            let mut session = SessionWithLog::spawn(Command::new("cat")).unwrap();
             let writer = StubWriter::default();
             session.set_log(writer.clone());
             session.send_line("Hello World").await.unwrap();
@@ -298,14 +302,13 @@ mod test {
         })
     }
 
-    #[cfg(feature = "async_log")]
-    #[cfg(feature = "async")]
+    #[cfg(all(feature = "async", feature = "log"))]
     #[test]
     fn deref() {
         use futures_lite::AsyncReadExt;
 
         futures_lite::future::block_on(async {
-            let mut session = crate::Session::spawn("cat").unwrap();
+            let mut session = crate::Session::spawn(Command::new("cat")).unwrap();
             let writer = StubWriter::default();
             session.set_log(writer.clone());
             session.send_line("Hello World").await.unwrap();
@@ -322,8 +325,7 @@ mod test {
         })
     }
 
-    #[cfg(feature = "async_log")]
-    #[cfg(feature = "async")]
+    #[cfg(all(feature = "async", feature = "log"))]
     #[test]
     fn log_bash() {
         futures_lite::future::block_on(async {
@@ -343,12 +345,11 @@ mod test {
         })
     }
 
-    #[cfg(feature = "async")]
-    #[cfg(feature = "async_log")]
+    #[cfg(all(feature = "async", feature = "log"))]
     #[test]
     fn log_read_line() {
         futures_lite::future::block_on(async {
-            let mut session = SessionWithLog::spawn("cat").unwrap();
+            let mut session = SessionWithLog::spawn(Command::new("cat")).unwrap();
             let writer = StubWriter::default();
             session.set_log(writer.clone());
             session.send_line("Hello World").await.unwrap();
