@@ -1,9 +1,13 @@
-use std::{
-    ops::{Deref, DerefMut},
-    process::Command,
-};
+//! This module contains a list of special Sessions that can be spawned.
 
 use crate::{error::Error, session::Found, Session};
+use std::ops::{Deref, DerefMut};
+
+#[cfg(unix)]
+use std::process::Command;
+
+#[cfg(windows)]
+use conpty::ProcAttr;
 
 /// Spawn a bash session.
 ///
@@ -11,7 +15,8 @@ use crate::{error::Error, session::Found, Session};
 ///
 /// If you wan't to use [Session::interact] method it is better to use just Session.
 /// Because we don't handle echoes here (currently). Ideally we need to.
-#[cfg(feature = "sync")]
+#[cfg(unix)]
+#[cfg(not(feature = "async"))]
 pub fn spawn_bash() -> Result<ReplSession, Error> {
     const DEFAULT_PROMPT: &str = "EXPECT_PROMPT";
     let mut cmd = Command::new("bash");
@@ -24,7 +29,7 @@ pub fn spawn_bash() -> Result<ReplSession, Error> {
         "PROMPT_COMMAND",
         "PS1=EXPECT_PROMPT; unset PROMPT_COMMAND; bind 'set enable-bracketed-paste off'",
     );
-    let mut bash = ReplSession::new(cmd, DEFAULT_PROMPT, Some("quit"))?;
+    let mut bash = ReplSession::spawn(cmd, DEFAULT_PROMPT, Some("quit"))?;
 
     // read a prompt to make it not available on next read.
     //
@@ -39,6 +44,7 @@ pub fn spawn_bash() -> Result<ReplSession, Error> {
 /// Spawn a bash session.
 ///
 /// It uses a custom prompt to be able to controll shell better.
+#[cfg(unix)]
 #[cfg(feature = "async")]
 pub async fn spawn_bash() -> Result<ReplSession, Error> {
     const DEFAULT_PROMPT: &str = "EXPECT_PROMPT";
@@ -52,7 +58,7 @@ pub async fn spawn_bash() -> Result<ReplSession, Error> {
         "PROMPT_COMMAND",
         "PS1=EXPECT_PROMPT; unset PROMPT_COMMAND; bind 'set enable-bracketed-paste off'",
     );
-    let mut bash = ReplSession::new(cmd, DEFAULT_PROMPT, Some("quit"))?;
+    let mut bash = ReplSession::spawn(cmd, DEFAULT_PROMPT, Some("quit"))?;
 
     // read a prompt to make it not available on next read.
     bash.expect_prompt().await?;
@@ -62,8 +68,44 @@ pub async fn spawn_bash() -> Result<ReplSession, Error> {
 
 /// Spawn default python's IDLE.
 pub fn spawn_python() -> Result<ReplSession, Error> {
-    let idle = ReplSession::new(Command::new("python"), ">>> ", Some("quit()"))?;
-    Ok(idle)
+    #[cfg(unix)]
+    {
+        let idle = ReplSession::spawn(Command::new("python"), ">>> ", Some("quit()"))?;
+        Ok(idle)
+    }
+    #[cfg(windows)]
+    {
+        // If we spawn it as ProcAttr::default().commandline("python") it will spawn processes endlessly....
+        let idle = ReplSession::spawn(ProcAttr::cmd("python".to_string()), ">>> ", Some("quit()"))?;
+        Ok(idle)
+    }
+}
+
+/// Spawn a powershell session.
+///
+/// It uses a custom prompt to be able to controll the shell.
+#[cfg(windows)]
+pub fn spawn_powershell() -> Result<ReplSession, Error> {
+    const DEFAULT_PROMPT: &str = "EXPECTED_PROMPT>";
+    let mut powershell = ReplSession::spawn(
+        ProcAttr::default().commandline(r"pwsh -NoProfile -NonInteractive -NoLogo".to_string()),
+        DEFAULT_PROMPT,
+        Some("exit"),
+    )?;
+    powershell.is_echo_on = true;
+
+    // https://stackoverflow.com/questions/5725888/windows-powershell-changing-the-command-prompt
+    powershell.execute(format!(
+        r#"function prompt {{ "{}"; return " " }}"#,
+        DEFAULT_PROMPT
+    ))?;
+
+    // https://stackoverflow.com/questions/69063656/is-it-possible-to-stop-powershell-wrapping-output-in-ansi-sequences/69063912#69063912
+    // https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_ansi_terminals?view=powershell-7.2#disabling-ansi-output
+    powershell.execute(r#"[System.Environment]::SetEnvironmentVariable("TERM", "dumb")"#)?;
+    powershell.execute(r#"[System.Environment]::SetEnvironmentVariable("TERM", "NO_COLOR")"#)?;
+
+    Ok(powershell)
 }
 
 /// A repl session: e.g. bash or the python shell:
@@ -82,12 +124,13 @@ pub struct ReplSession {
 }
 
 impl ReplSession {
-    pub fn new<P: AsRef<str>, Q: AsRef<str>>(
+    #[cfg(unix)]
+    pub fn spawn<P: AsRef<str>, Q: AsRef<str>>(
         cmd: Command,
         prompt: P,
         quit_command: Option<Q>,
     ) -> Result<Self, Error> {
-        let session = Session::spawn_cmd(cmd)?;
+        let session = Session::spawn(cmd)?;
         let is_echo_on = session.get_echo()?;
         let prompt = prompt.as_ref().to_owned();
         let quit_command = quit_command.map(|q| q.as_ref().to_owned());
@@ -100,6 +143,24 @@ impl ReplSession {
         })
     }
 
+    #[cfg(windows)]
+    pub fn spawn<P: AsRef<str>, Q: AsRef<str>>(
+        attr: crate::ProcAttr,
+        prompt: P,
+        quit_command: Option<Q>,
+    ) -> Result<Self, Error> {
+        let session = Session::spawn(attr)?;
+        let prompt = prompt.as_ref().to_owned();
+        let quit_command = quit_command.map(|q| q.as_ref().to_owned());
+
+        Ok(Self {
+            prompt,
+            session,
+            quit_command,
+            is_echo_on: false,
+        })
+    }
+
     /// Get a size in bytes of a prompt, may be usefull for triming it.
     pub fn prompt_len(&self) -> usize {
         self.prompt.as_bytes().len()
@@ -108,24 +169,12 @@ impl ReplSession {
 
 impl ReplSession {
     /// Block until prompt is found
-    #[cfg(feature = "sync")]
+    #[cfg(not(feature = "async"))]
     pub fn expect_prompt(&mut self) -> Result<(), Error> {
         self._expect_prompt()?;
         Ok(())
     }
 
-    #[cfg(feature = "sync")]
-    pub fn _expect_prompt(&mut self) -> Result<Found, Error> {
-        let prompt = self.prompt.clone();
-        self.expect(prompt)
-    }
-
-    #[cfg(feature = "async")]
-    pub async fn _expect_prompt(&mut self) -> Result<Found, Error> {
-        let prompt = self.prompt.clone();
-        self.expect(prompt).await
-    }
-
     /// Block until prompt is found
     #[cfg(feature = "async")]
     pub async fn expect_prompt(&mut self) -> Result<(), Error> {
@@ -133,35 +182,39 @@ impl ReplSession {
         Ok(())
     }
 
+    #[cfg(not(feature = "async"))]
+    fn _expect_prompt(&mut self) -> Result<Found, Error> {
+        let prompt = self.prompt.clone();
+        self.expect(prompt)
+    }
+
+    #[cfg(feature = "async")]
+    async fn _expect_prompt(&mut self) -> Result<Found, Error> {
+        let prompt = self.prompt.clone();
+        self.expect(prompt).await
+    }
+
     /// Send a command to a repl and verifies that it exited.
     /// Returning it's output.
-    #[cfg(feature = "sync")]
+    #[cfg(not(feature = "async"))]
     pub fn execute<S: AsRef<str> + Clone>(&mut self, cmd: S) -> Result<Vec<u8>, Error> {
-        self.send_line(cmd.clone())?;
-        if self.is_echo_on {
-            self.expect(cmd.as_ref())?;
-        }
-
+        self.send_line(cmd)?;
         let found = self._expect_prompt()?;
-        Ok(found.before_match().to_vec())
+        Ok(found.before().to_vec())
     }
 
     /// Send a command to a repl and verifies that it exited.
     #[cfg(feature = "async")]
     pub async fn execute<S: AsRef<str> + Clone>(&mut self, cmd: S) -> Result<Vec<u8>, Error> {
-        self.send_line(cmd.clone()).await?;
-        if self.is_echo_on {
-            self.expect(cmd.as_ref()).await?;
-        }
-
+        self.send_line(cmd).await?;
         let found = self._expect_prompt().await?;
-        Ok(found.before_match().to_vec())
+        Ok(found.before().to_vec())
     }
 
     /// Sends line to repl (and flush the output).
     ///
     /// If echo_on=true wait for the input to appear.
-    #[cfg(feature = "sync")]
+    #[cfg(not(feature = "async"))]
     pub fn send_line<S: AsRef<str>>(&mut self, line: S) -> Result<(), Error> {
         self.session.send_line(line.as_ref())?;
         if self.is_echo_on {
@@ -196,7 +249,7 @@ impl ReplSession {
     }
 }
 
-#[cfg(feature = "sync")]
+#[cfg(not(feature = "async"))]
 impl Drop for ReplSession {
     fn drop(&mut self) {
         if let Some(quit_command) = self.quit_command.clone() {
